require_relative 'project'
require_relative 'die'
require_relative 'funding_round'
require_relative 'pledge_pool'

module FundRaise
  
class FundRequest
  attr_accessor :name
  attr_reader :projects_ar
  
  def initialize (mylistname)
    @name = mylistname
    @projects_ar = []
  end
      
  def add_project(project)
    @projects_ar.push(project)
  end
    
  def load_projects(from_file)
    File.readlines(from_file).each do |line|
      add_project(Project.from_csv(line))
    end
  end

  def save_underfunded_projects(to_file="underfunded_projects.txt")
    File.open(to_file, "w") do |file|
      file.puts "#{@name} Under-funded projects:"

      @projects_ar.sort.each do |p|
        if !p.fully_funded?
          file.puts underfunded_entry(p)
        end
      end
    end
  end
  
  def underfunded_entry(project)
    formatted_name = project.name.ljust(20, '.')
    "#{formatted_name} $#{project.funding_left} needed"
  end
  
  def request_funding(rounds)
    PledgePool::print_pledges_available
    puts "\nProject List #{@name}:"
    puts @projects_ar.sort
    
    1.upto(rounds) do |round|
      puts "\nRound #{round}:"
      @projects_ar.each do |project|
        puts project
        FundingRound.one_round(project)
        puts "#{project}\n\n"
      end
    end
  end
  
  def total_pledges
    @projects_ar.reduce(0) do |sum, project|
      sum + project.total_pledge_points
    end
  end
  
  def print_stats
    puts "\n#{@name}'s funding:"
    
    puts "$#{total_pledges} in total pledges."
    @projects_ar.sort.each do |p|
      puts "\n#{p.name}'s pledges:"
      p.each_given_pledge do |t|
        puts "$#{t.amount} in #{t.name} pledges."
      end
    end
    
    fully, not_fully = @projects_ar.partition { |project| project.fully_funded? }
    
    puts "\nThese projects are fully funded:"
    fully.sort.each do |p|
      puts "#{p.name}"
    end
      
    puts "\n#{not_fully.size} projects are not fully funded:"
    not_fully.sort.each do |p|
      puts "#{p.name}  $#{p.current_funding}"
    end
    
    puts "\n#{not_fully.size} projects need your help:"
    not_fully.sort.each do |p|
      formatted_name = p.name.ljust(20, '.')
      puts "#{formatted_name} $#{p.funding_left} funds needed"
    end
    
  end
  
  def to_s
    "There are #{@projects_ar.size} projects in #{@name}."
  end
end

if __FILE__ == $0

  project1 = Project.new("Project ABC", 5000, 1000)
  project2 = Project.new("Project LMN", 3000, 500)
  project3 = Project.new("Project XYZ", 75, 25)

  my_project = FundRequest.new("VC-Friendly Start-up Projects.")

  my_project.add_project(project1)
  my_project.add_project(project2)
  my_project.add_project(project3)
  
  my_project.request_funding

end

end