require_relative 'pledge_pool'
require_relative 'fundable'

module FundRaise
class Project
  include Fundable
  
  attr_accessor :name
  attr_reader :current_funding, :target_funding
  
  def initialize (name, target=1000, funding=0)
    @name = name
    @current_funding = funding
    @target_funding = target
    @pledges = Hash.new(0)
  end

  def total_pledge_points
    @pledges.values.reduce(0, :+)
  end

  def pledge_received(pledge)
    @pledges[pledge.name] += pledge.amount
    add_funds(pledge.amount, true)
    puts "#{@name} received a #{pledge.name} pledge worth $#{pledge.amount}."
    puts "#{@name}'s pledges: #{@pledges}"
  end

  def each_given_pledge
    @pledges.each do |name, amount|
      pledge = Pledge.new(name, amount)
      yield pledge
    end
  end
  
  def time
    current_time = Time.new
    current_time.strftime("%A, %B %d, %Y")
  end

  def <=>(other_project)
    funding_left <=> other_project.funding_left
  end
  
  def self.from_csv(string)
    name, target, funding = string.split(',')
    Project.new(name, Integer(target), Integer(funding))
  end
  
  def to_csv
    "#{@name}, #{@target_funding}, #{@current_funding}"
  end
  
  def to_s
    "#{@name} has $#{@current_funding} in funding towards a goal of $#{@target_funding}."
  end

end

if __FILE__ == $0

project1 = Project.new("Project LMN", 3000, 500)
project2 = Project.new("Project XYZ", 255, 25)
project3 = Project.new("Project PTR", 100, 110)

projects_ar = [project1, project2]

puts "There are #{projects_ar.size} projects in the array."
projects_ar.each do |project|
  puts project
end
puts project1.remove_funds
puts project2.add_funds(24)

projects_ar.each do |project|
  puts project
end

projects_ar.each do |project|
  if project.current_funding < 90
    puts "DELETE #{project.name}"
    projects_ar.delete(project)
  end  
end
projects_ar.push(project3)

puts "\nThere are #{projects_ar.size} projects in the array."
projects_ar.each do |project|
  puts project
end

end
end